// Unless you want to include all components, you must set $include-default to false
// IF you set this to true, you can also remove lines 10 to 38 of this file
$include-default: false;

// Insert your custom variables here.
$base-color: #bfbfff;
$base-gradient: 'flat' !default;
$font-family: 'Neuton';
$font-size: 14px; 

//$form-field-background-color: #cfcfcf;
//$form-field-color: #00002f;
//$form-label-background-color: #1f1f1f;
//$form-label-color: #dfdfff;
//$form-radio-background-color: #efdfbf;
//$form-radio-color: #00002f;
//$fieldset-header-line-height: 20px;

//$panel-body-background-color: #ffefdf;
$panel-body-font-size: $font-size;
$form-font-size: $font-size;
$form-label-font-size: $font-size;
$form-radio-font-size: $font-size;
$fieldset-font-size: $font-size;

.x-grid-cell-inner {
  overflow: auto !important;
  white-space: normal !important;
}

//---------------------------------------------------------

$progress-bar-base-color: $base-color !default;
$progress-height: 20px !default;

//borders
$progress-border-color: adjust-color($progress-bar-base-color, $hue: 0deg, $saturation: -3.08%, $lightness: -23.725%) !default;
$progress-border-width: 1px !default;
$progress-border-radius: 2px !default;

//backgrounds
$progress-background-color: adjust-color($progress-bar-base-color, $hue: 120deg, $saturation: -11.37%, $lightness: 7.451%) !default;

//bar
$progress-bar-background-color: adjust-color($progress-bar-base-color, $hue: 240deg, $saturation: 8.187%, $lightness: -17.647%) !default;

//text
$progress-text-color-front: #fff !default;
$progress-text-color-back: adjust-color($progress-bar-base-color, $hue: 0deg, $saturation: -10.895%, $lightness: -43.725%) !default;
$progress-text-text-align: center !default;
$progress-text-font-size: ceil($font-size * .9) !default;
$progress-text-font-weight: normal !default;

//---------------------------------------------------------
@import 'ext4/default/all';

// You may remove any of the following modules that you
// do not use in order to create a smaller css file.
@include extjs-boundlist;
@include extjs-button;
@include extjs-btn-group;
@include extjs-datepicker;
@include extjs-colorpicker;
@include extjs-menu;
@include extjs-grid;
@include extjs-form;
    @include extjs-form-field;
    @include extjs-form-fieldset;
    @include extjs-form-file;
    @include extjs-form-checkboxfield;
    @include extjs-form-checkboxgroup;
    @include extjs-form-triggerfield;
    @include extjs-form-htmleditor;
@include extjs-panel;
@include extjs-qtip;
@include extjs-slider;
@include extjs-progress;
@include extjs-toolbar;
@include extjs-window;
@include extjs-messagebox;
@include extjs-tabbar;
@include extjs-tab;
@include extjs-tree;
@include extjs-drawcomponent;
@include extjs-viewport;
